/* Copyright 1993, 1994, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* painter.c - use the painter's algorithm to draw a solid polygon 
 *	in front of a line grid
 *
 *	<r> Key		- rotate polygon along X axis
 *	Escape Key	- exit program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initgfx( GLvoid );
GLvoid  keyboard( GLubyte, GLint, GLint );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static GLboolean doRotate = GL_FALSE;

void
main( int argc, char *argv[] )
{
	GLsizei width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( 0, height / 4 );
	glutInitWindowSize( (width / 2) - 4, height / 2 );
	glutInitDisplayMode( GLUT_RGBA );
	glutCreateWindow( argv[0] );

	initgfx();

	glutReshapeFunc( reshape );
	glutKeyboardFunc( keyboard );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrates painter's algorithm\n\n"
		"<r> Key		- rotate polygon around X axis\n"
		"Escape Key		- exit the program\n\n",
		progname);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 1.0, 1.0 );
	glShadeModel( GL_FLAT );
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble    aspect;

	glViewport( 0, 0, width, height );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	aspect = (GLdouble) width / (GLdouble) height;
	gluPerspective( 45.0, aspect, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case 'r':
		doRotate = !doRotate;
		glutPostRedisplay();
		break;
	case KEY_ESC:	/* Exit when the Escape key is pressed */
		exit(0);
	}
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat    whiteColor[] = { 1.0, 1.0, 1.0 };
	static GLfloat    purpleColor[] = { 0.8, 0.0, 1.0 };

	glClear( GL_COLOR_BUFFER_BIT );

	glPushMatrix();
		/* Move origin between the near and far clipping planes */
		glTranslatef( 0.0, 0.0, -4.0 );

		/* Draw a grid in the x-y plane centered at the origin */
		glColor3fv( whiteColor );
		WireGrid();

		glColor3fv( purpleColor );

		/* Draw a rectangle in front of the grid */
		glTranslatef( 0.0, 0.0, 0.1 );

		if (doRotate)
			glRotatef( -45.0, 1.0, 0.0, 0.0 );

		glRectf( -0.5, -0.5, 0.5, 0.5 );
	glPopMatrix();

	glFlush();
}
