/*
 * Copyright 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* order.c - illustrate what happens when the order
 *      of the modeling transformations is
 *
 *            glTranslate( ... );
 *            glRotate( ... );
 * 	versus 
 *            glRotate( ... );
 *            glTranslate( ... );
 *
 *
 *	<o> key		- toggle order of transformations
 *	Escape key	- exit program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initgfx( GLvoid );
GLvoid  keyboard( GLubyte, GLint, GLint );
GLvoid  drawScene( GLvoid );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static GLboolean rotateFirst = GL_TRUE;

void
main( int argc, char *argv[] )
{
	GLsizei width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width / 4, height / 4 );
	glutInitWindowSize( (width / 2) - 4, height / 2 );
	glutInitDisplayMode( GLUT_RGBA );
	glutCreateWindow( argv[0] );

	initgfx();

	glutKeyboardFunc( keyboard );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glMatrixMode( GL_PROJECTION );
	glOrtho( -2.0, 2.0, -2.0, 2.0, -1.0, 1.0 );
	glMatrixMode( GL_MODELVIEW );

	glutMainLoop();
}

void
printHelp( char *progname )
{
	fprintf(stdout, 
		"\n%s - demonstrates the effect of transformation order\n\n"
		"<o> key		- toggle order of transformations\n"
		"Escape Key		- exit the program\n\n",
		progname);

	fprintf(stdout, "order is Rotate-Translate\n");
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 1.0, 1.0 );
	glShadeModel( GL_FLAT );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case 'o':	/* toggle transformation order */
		rotateFirst = !rotateFirst;
		fprintf(stdout, "order is %s\n",
			(rotateFirst?"Rotate-Translate":"Translate-Rotate"));
		glutPostRedisplay();
		break;
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
drawScene( GLvoid )
{
	glClear( GL_COLOR_BUFFER_BIT );

	glPushMatrix();
		if ( rotateFirst ) {
			/* Do a rotation first, then a translation */
			glRotatef( 60.0, 0.0, 0.0, 1.0 );
			glTranslatef( 1.0, 0.0, 0.0 );
		} else {
			/* Do a translation first, then a rotation */
			glTranslatef( 1.0, 0.0, 0.0 );
			glRotatef( 60.0, 0.0, 0.0, 1.0 );
		}

		Checkerboard();

		/* Draw X-Y axis -- X is red, Y is green */
		XYaxes();

	glPopMatrix();

	glFlush();
}
