/*
 * Copyright 1993, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* robot_arm.c - draw a robot arm by composing modeling 
 *	transformations
 *
 *	Escape Key	- exit program
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initgfx( GLvoid );
GLvoid  keyboard( GLubyte, GLint, GLint );
GLvoid  drawScene( GLvoid );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

void
main( int argc, char *argv[] )
{
	GLsizei width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( 0, height / 4 );
	glutInitWindowSize( (width / 2) - 4, height / 2 );
	glutInitDisplayMode( GLUT_RGBA );
	glutCreateWindow( argv[0] );

	initgfx();

	glutKeyboardFunc( keyboard );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, (GLdouble) width/ (GLdouble)height, 1.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	
	glutMainLoop();
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - renders a robot arm\n\n"
		"Escape Key		- exit the program\n\n",
		progname);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glShadeModel( GL_FLAT );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat    upperArmColor[] = { 1.0, 0.0, 0.0 };
	static GLfloat    lowerArmColor[] = { 0.8, 0.5, 0.5 };
	
	glClear( GL_COLOR_BUFFER_BIT );

	glPushMatrix();
		/* translate into the viewing volume */
		glTranslatef( 0.0, 0.0, -8.0 );

		XYaxes();

		/* Draw the upper arm with the shoulder
		 * on the Z axis */

		/* Step 1: move to the center of the upper arm */
		glTranslatef( 1.0, 0.0, 0.0 ); 

		/* Step 2: draw the upper arm */
		glColor3fv( upperArmColor );
		WireBox( 2.0, 0.4, 1.0 );

		/* Draw the lower arm at the end of the upper arm and 
		 * rotate it 45 degrees */

		/* Step 3: move to the elbow */
		glTranslatef( 1.0, 0.0, 0.0 );

		/* Step 4: rotate the lower arm 45 degrees */
		glRotatef( 45.0, 0.0, 0.0, 1.0 );

		/* Step 5: move to the center of the lower arm */
		glTranslatef( 1.0, 0.0, 0.0 );

		/* Step 6: draw the lower arm */
		glColor3fv( lowerArmColor );
		WireBox( 2.0, 0.4, 1.0 );
	glPopMatrix();

	glFlush();
}
