/*
 * Copyright 1993, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*  checkerboard.c  - open a window, clear the background, and 
 *	render a checkerboard composed of flat shaded rectangles.
 *
 *	Escape key	- exit program
 */

#include <GL/gl.h>
#include <GL/glut.h>

#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initgfx( GLvoid );
GLvoid	keyboard( GLubyte, GLint, GLint );
GLvoid  drawScene( GLvoid );

void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

void
main( int argc, char *argv[] )
{
	GLsizei width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width / 4, height / 4 );
	glutInitWindowSize( width / 2, height / 2 );
	glutInitDisplayMode( GLUT_RGBA );
	glutCreateWindow( argv[0] );

	initgfx();

	glutKeyboardFunc( keyboard );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - renders a checkerboard\n\n"
		"Escape Key		- exit the program\n\n",
		progname);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 1.0, 1.0 );

	glOrtho( -2.0, 2.0, -2.0, 2.0, -1.0, 1.0 );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid
drawScene( GLvoid )
{
	GLfloat		x, y;

	static GLfloat	redColor[] = { 1.0, 0.0, 0.0 };
	static GLfloat	blackColor[] = { 0.0, 0.0, 0.0 };

	/* alternate between red and black rectangles*/
	GLboolean	useRed = GL_TRUE;  

	glClear( GL_COLOR_BUFFER_BIT );

	/* Draw a checkerboard composed of 0.1 by 0.1 rectangles 
	 * (For convenience, this code as been added to the class 
	 * library as Checkerboard()).
	 */

	for ( x = -1.0; x < 1.0; x += 0.1 ) {
		useRed = !useRed;
		for ( y = -1.0; y < 1.0; y += 0.1 ) {
			if ( useRed )
				glColor3fv( redColor );
			else
				glColor3fv( blackColor );
			glRectf( x, y, x + 0.1, y + 0.1 );
			useRed = !useRed;
		}
	}

	glFlush();
}
