/*
 * (c) Copyright 1993, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "fonts.h"

GLenum tkCreateStrokeFont(GLuint fontBase)
{
    GLint mode, i, j;

    for (i = 0; strokeFont[i][0] != END_OF_LIST; i++) {
	glNewList(fontBase+(unsigned int)strokeFont[i][0], GL_COMPILE);
	for (j = 1; mode = strokeFont[i][j]; j += 3) {
	    if (mode == FONT_BEGIN) {
		glBegin(GL_LINE_LOOP);
		glVertex2f((float)strokeFont[i][j+1]*STROKE_SCALE,
			   (float)strokeFont[i][j+2]*STROKE_SCALE);
	    } else if (mode == FONT_NEXT) {
		glVertex2f((float)strokeFont[i][j+1]*STROKE_SCALE,
			   (float)strokeFont[i][j+2]*STROKE_SCALE);
	    } else if (mode == FONT_END) {
		glVertex2f((float)strokeFont[i][j+1]*STROKE_SCALE,
			   (float)strokeFont[i][j+2]*STROKE_SCALE);
		glEnd();
	    } else if (mode == FONT_ADVANCE) {
		glTranslatef((float)strokeFont[i][j+1]*STROKE_SCALE,
			     (float)strokeFont[i][j+2]*STROKE_SCALE, 0.0);
		break;
	    }
	}
	glEndList();
    }
    return GL_TRUE;
}

GLenum tkCreateOutlineFont(GLuint fontBase)
{
    GLint mode, i, j;

    for (i = 0; outlineFont[i][0] != END_OF_LIST; i++) {
	glNewList(fontBase+(unsigned int)outlineFont[i][0], GL_COMPILE);
	for (j = 1; mode = outlineFont[i][j]; j += 3) {
	    if (mode == FONT_BEGIN) {
		glBegin(GL_LINE_STRIP);
		glVertex2f((float)outlineFont[i][j+1]*OUTLINE_SCALE,
			   (float)outlineFont[i][j+2]*OUTLINE_SCALE);
	    } else if (mode == FONT_NEXT) {
		glVertex2f((float)outlineFont[i][j+1]*OUTLINE_SCALE,
			   (float)outlineFont[i][j+2]*OUTLINE_SCALE);
	    } else if (mode == FONT_END) {
		glVertex2f((float)outlineFont[i][j+1]*OUTLINE_SCALE,
			   (float)outlineFont[i][j+2]*OUTLINE_SCALE);
		glEnd();
	    } else if (mode == FONT_ADVANCE) {
		glTranslatef((float)outlineFont[i][j+1]*OUTLINE_SCALE,
			     (float)outlineFont[i][j+2]*OUTLINE_SCALE, 0.0);
		break;
	    }
	}
	glEndList();
    }
    return GL_TRUE;
}

GLenum tkCreateFilledFont(GLuint fontBase)
{
    GLint mode, i, j;

    for (i = 0; filledFont[i][0] != END_OF_LIST; i++) {
	glNewList(fontBase+(unsigned int)filledFont[i][0], GL_COMPILE);
	for (j = 1; mode = filledFont[i][j]; j += 3) {
	    if (mode == FONT_BEGIN) {
		glBegin(GL_TRIANGLE_STRIP);
		glVertex2f((float)filledFont[i][j+1]*FILLED_SCALE,
			   (float)filledFont[i][j+2]*FILLED_SCALE);
	    } else if (mode == FONT_NEXT) {
		glVertex2f((float)filledFont[i][j+1]*FILLED_SCALE,
			   (float)filledFont[i][j+2]*FILLED_SCALE);
	    } else if (mode == FONT_END) {
		glVertex2f((float)filledFont[i][j+1]*FILLED_SCALE,
			   (float)filledFont[i][j+2]*FILLED_SCALE);
		glEnd();
	    } else if (mode == FONT_ADVANCE) {
		glTranslatef((float)filledFont[i][j+1]*FILLED_SCALE,
			     (float)filledFont[i][j+2]*FILLED_SCALE, 0.0);
		break;
	    }
	}
	glEndList();
    }
    return GL_TRUE;
}

void DrawStr(GLuint base, char *str)
{

    glPushAttrib(GL_LIST_BIT);
    glListBase(base);
    glCallLists(strlen(str), GL_UNSIGNED_BYTE, (unsigned char *)str);
    glPopAttrib();
}

/**************************************************************************
 *   RenderBitmapFontString() - Print an ASCII text string in given font
 **************************************************************************/

GLvoid
RenderBitmapFontString( void *font, char *string )
{
	int i;
	int len = (int) strlen(string);
	for (i = 0; i < len; i++) {
		glutBitmapCharacter(font, string[i]);
	}
}
